# 勾配上昇法で正規分布の対数尤度を最適化する

# データを乱数で作成（標準正規分布）
x=rnorm(100);n=length(x)

# 対数尤度を目的関数として最大化する
f<-function(mu,sigma){
  return(-log(2*pi*sigma^2)*n/2-sum((x-mu)^2)/sqrt(2*sigma^2))
}

# 正規分布のパラメータの初期値
MU=1;SIGMA=2

# 反復計算回数
ite=1000

# 学習率
eta=0.001

# 差分メッシュ
h=0.01

for(l in 1:ite){
  # パラメータの保存
  param=c(MU,SIGMA)

  # パラメータの更新：MU
  MU=MU+eta*sum(x-MU)/(SIGMA^2)

  # パラメータの更新：SIGMA
  SIGMA=SIGMA+eta*(-n/(2*SIGMA^2)+sum((x-MU)^2)/(2*SIGMA^4))

  # 対数尤度を計算
  print(f(MU,SIGMA))
}

# 最尤推定量：MU
mean(x)

# 最尤推定量：SIGMA
var(x)
