# 準ニュートン法（Broyden Quasi-Newton Algorithm）と正規方程式の比較

# 目的変量yと説明変量x1,x2の入ったデータ
data=data.frame(num=1:20,y=c(167,167.5,168.4,172,155.3,151.4,163,174,168,160.4,164.7,171,162.6,164.8,163.3,167.6,169.2,168,167.4,172),x1=c(84,87,86,85,82,87,92,94,88,84.9,78,90,88,87,82,84,86,83,85.2,82),x2=c(61,55.5,57,57,50,50,66.5,65,60.5,49.5,49.5,61,59.5,58.4,53.5,54,60,58.8,54,56))

# 目的変量のベクトル
Y=as.numeric(data$y)

# 説明変量の行列
X=as.matrix(data[,colnames(data) %in% c("x1","x2")])

# 説明変量の行列（更新後）
XX=cbind(rep(1,nrow(X)),X)

# Broyden Quasi-Newton Algorithm

# 行列形式
f<-function(x){
  return(c(-2*t(XX)%*%Y+2*t(XX)%*%XX%*%x))
}

# 偏回帰係数のパラメータの初期値
X=rep(1,ncol(XX))

# 反復計算回数
ite=10000

# ヤコビアンの近似行列の初期値
H=diag(f(X))

# 学習率
eta=0.1

for(l in 1:ite){
  # 全誤差の合計が10^(-9)以下になったら計算打ち止め
  if(sum(abs(f(X)))>10^(-9)){
    # 更新前のパラメータ（偏回帰係数）を保存
    X_pre=X

    # パラメータの更新
    X=X-eta*H%*%f(X)

    # 更新前と更新後のパラメータの差分
    s=X-X_pre

    # 最小化したいコストの差分
    y=f(X)-f(X_pre)

    # 近似的なヤコビアン行列の作成
    H=H+((s-H%*%y)/as.numeric(t(s)%*%H%*%y))%*%t(s)%*%H

    print((f(X)))
  }
}

# 正規方程式を直接解く場合
solve(t(XX)%*%XX)%*%t(XX)%*%Y
