# multiple regression

library(dplyr)

# 例1
data=data.frame(num=1:20,y=c(167,167.5,168.4,172,155.3,151.4,163,174,168,160.4,164.7,171,162.6,164.8,163.3,167.6,169.2,168,167.4,172),x1=c(84,87,86,85,82,87,92,94,88,84.9,78,90,88,87,82,84,86,83,85.2,82),x2=c(61,55.5,57,57,50,50,66.5,65,60.5,49.5,49.5,61,59.5,58.4,53.5,54,60,58.8,54,56))

# 平均と標準偏差のデータ
Expect_and_sd_data=data.frame(value=c("mean","sd"),y=c(mean(data$y),sd(data$y)),x1=c(mean(data$x1),sd(data$x1)),x2=c(mean(data$x2),sd(data$x2)))

Data=data %>% select(x1,x2,y)

# 偏差積和行列
A=array(0,dim=c(3,3))
for(i in 1:ncol(A)){
  for(j in 1:nrow(A)){
    A[i,j]=sum((Data[,i]-mean(Data[,i]))*(Data[,j]-mean(Data[,j])))
  }
}
part_of_inverse_A=solve(A[1:2,1:2])

# 偏回帰係数
b1=sum(A[1:2,3]*part_of_inverse_A[1,1:2])
b2=sum(A[1:2,3]*part_of_inverse_A[2,1:2])
b0=mean(data$y)-b1*mean(data$x1)-b2*mean(data$x2)
print(paste0("Y=",signif(b0,digits=4),signif(b1,digits=4),"x1+",signif(b2,digits=4),"x2"))

# 重回帰式の有意性の検定

# 全変動
ayy=A[3,3]

# 残差の変動
EV=sum((Data$y-(b0+b1*Data$x1+b2*Data$x2))^2)

# 回帰による変動
RV=sum(((b0+b1*Data$x1+b2*Data$x2)-mean(b0+b1*Data$x1+b2*Data$x2))^2)

# 全変動の値＝EV＋RVとなっているかどうか
if(signif(ayy,digits=6)==signif(EV+RV,digits=6)){
  print("your answer is right!")
}

# 重相関係数
R=(1-EV/(EV+RV))^(1/2)

# 寄与率（propotion）
R^2

#標準変量の追加
data=data%>%mutate(Y=(y-mean(y))/sd(y),X1=(x1-mean(x1))/sd(x1),X2=(x2-mean(x2))/sd(x2))

plot(data$Y,ylim=c(-5,5),type="p",col=1,ylab="身長")
plot(data$X1,ylim=c(-5,5),type="p",col=2,ylab="胸囲")
plot(data$X2,ylim=c(-5,5),type="p",col=3,ylab="体重")

# 各特徴量が標準化された時の偏回帰係数（切片は0になる）
B1=b1*((A[1,1]/ayy)^(1/2));B2=b2*((A[2,2]/ayy)^(1/2))

# 回帰方程式のプロット
print(paste0("Y=",signif(B1,digits = 4),"X1+",signif(B2,digits=4),"X2"))

# 不偏分散
V_R=RV/(ncol(Data)-1);V_e=EV/(nrow(Data)-(ncol(Data)-1)-1)

# f検定量
static_F=V_R/V_e

# f分布の密度関数の作成
F_dis=function(n1,n2,x){
  y<-(gamma((n1+n2)/2)*((n1/n2)^(n1/2))*(x^((n1/2)-1))*((1+(n1*x)/n2)^(-(n1+n2)/2)))/(gamma(n1/2)*gamma(n2/2))
  return(y)
}

# 自由度
n1=(ncol(Data)-1);n2=(nrow(Data)-(ncol(Data)-1)-1)

# f分布のプロットデータ
F_dis_data=data.frame(num=seq(0.01,10,by=0.01)) %>% mutate(value=F_dis(n1,n2,num) ,test=df(num,n1,n2))

# f分布のプロット
plot(F_dis_data$num,F_dis_data$value,type="l")

# 実際にパッケージの値と比べてみてください
# 0.01のところはF_dis_dataのnumの値、valueは出力値になります
df(0.01,n1,n2)

# F分布の確率点
alpha=0.05

x_point=qf((1-alpha),2,17)

# H:このモデルは目的変量の推定の役に立たない
if(x_point<static_F){
  print(paste0("危険率",100*alpha,"%を許すならば有意である"))
}
