# 冠状動脈性心疾患による死亡者数

# 最小二乗法

# 死亡者数
DEATH=c(32,104,206,186,102,2,12,28,28,31)

# 観測者数
PERSON=c(52407,43248,28612,12663,5317,18790,10673,5710,2585,1462)

MR=100000*DEATH/PERSON

# 年齢階層（0：35歳未満、1：35～44歳、2：45～54歳、3：55～64歳、4：65～74歳、5：75～84歳）
X3=c(1:5,1:5)
X4=X3^2

# 喫煙X3、非喫煙0
X5=c(1:5,rep(0,5))

# 喫煙1、非喫煙0
X2=c(rep(1,5),rep(0,5))

# 特徴量行列
X=cbind(X2,X3,X4,X5)

# 最小二乗法
beta=solve(t(X)%*%X)%*%t(X)%*%log(MR)

# 予測値
pre_MR=exp(X%*%beta)

# 予測値（MR）
plot(c(1:length(MR)),MR,xlab="num",ylab="values",ylim=c(min(c(MR,pre_MR)),max(c(MR,pre_MR))),col=2,pch=19)

par(new=T)

plot(c(1:length(MR)),MR,xlab="num",ylab="values",ylim=c(min(c(MR,pre_MR)),max(c(MR,pre_MR))),col=3,pch=1)

# 喫煙者
exp(sum(beta))

# 非喫煙者
exp(sum(beta*c(0,1,1,0)))

# ポアソン回帰

# 死亡者数
Y=DEATH

# 特徴量行列
X=cbind(rep(1,length(X2)),X)

# 反復計算回数
times=1000

# 回帰係数の計算履歴を保存するデータを作成
beta <- matrix(0, ncol = ncol(X), nrow = times)

# 初期値
beta[1, ] <- rep(1,ncol(X))

for(m in 2:times){
  lambda <- exp(X %*% beta[m - 1, ])
  W <- diag(lambda[, 1])
  XtWX <- t(X) %*% W %*% X
  D=diag(1/lambda[,1])
  U <- t(X)%*%W%*%D%*%(Y-lambda[,1])
  beta[m, ] <- beta[m - 1, ] + solve(XtWX) %*% (U)
}

# 回帰係数
params1=beta[nrow(beta),]

# 予測値
predict1=exp(X%*%params1)

# 実測値のプロット
plot(c(1:length(Y)),(Y),type="p",col=2,xlab="num",ylab="values",ylim=c(min((Y)),max((Y))),pch=19)

par(new=T)

# 予測値のプロット
plot(c(1:length(Y)),predict1,type="p",col=3,xlab="num",ylab="values",ylim=c(min((Y)),max((Y))),pch=1)

# H：回帰係数=0の統計量
Z=params1/sqrt(diag(solve(t(X)%*%W%*%X)))

X_star=sqrt(W)%*%X
H_star=X_star%*%solve(t(X_star)%*%X_star)%*%t(X_star)
d=(Y*log(Y/predict1)-(Y-predict1))
L_y_y=(Y*log(Y/mean(Y))-(Y-mean(Y)))

# 尤離度決定係数
R2_dev=1-sum(d)/sum(L_y_y)

# ピアソン残差
r_pi=(Y-predict1)/sqrt(predict1)

# 標準ピアソン残差
S_pi=r_pi/sqrt(1-diag(H_star))

# 尤離度残差
r_di=sign(Y-predict1)*sqrt(d)

# 標準尤離度残差
S_di=r_di/sqrt(1-diag(H_star))

# 結果まとめ
res=data.frame(Y=Y,Y_hat=predict1,r_pi=r_pi,r_di=r_di,S_pi=S_pi,S_di=S_di)
