# ロジスティック回帰・プロビット回帰・補対数対数回帰

library(dplyr)

# 不妊者と非不妊者について調べたデータ
data(infert)

# 目的変量（不妊：1、対照：0）
Y=infert$case

# 特徴量行列　age：年齢、induced：それまでの人工妊娠中絶回数（2は2回以上）、
# 　　　　　　spontaneous：それまでの自然流産回数（2は2回以上）
X=cbind(rep(1,nrow(infert)),infert[,colnames(infert) %in% c("age","induced","spontaneous")])

X=as.matrix(X)

# モデル選択　1：ロジスティック回帰、2：プロビット、3：補対数対数モデル
model=3

# 初期値
alpha=rep(0.01,(ncol(X)))

# 反復計算回数
ite=1000

# 回帰係数保存データ
alpha_beta_data=array(0,dim=c(ite,ncol(X)))

for(j in 1:ite){
  if(model==1){
    p=1/(1+exp(-(X%*%alpha)))
    D=diag(c(1/(p*(1-p))))
    V=diag(c(p*(1-p)))
  }else{
    if(model==2){
      p=pnorm(X%*%alpha)
      D=diag(c(1/dnorm(X%*%alpha)))
      V=diag(1/c(p*(1-p)/(dnorm(X%*%alpha)^2)))
    }else{
      p=1-exp(-exp(X%*%alpha))
      V=diag(c(((1-p)*log(1-p))^2/(p*(1-p))))
      D=diag(c(1/(-(1-p)*log(1-p))))
    }
  }

  mat=solve(t(X)%*%V%*%(X))%*%t(X)%*%V%*%D%*%(Y-p)
  alpha=alpha+mat
  alpha_beta_data[j,]=alpha
}

res=data.frame(predict=p,Y=Y)

# 対数尤度
log_lik=sum(Y*log(ifelse(p>0,p,1))+(1-Y)*log(ifelse(1-p>0,1-p,1)))

AIC=-2*log_lik+2*(length(alpha))
BIC=-2*log_lik+length(alpha)*log(nrow(X))
