# 細胞の死亡数に対する2つの薬剤の濃度（薬剤1の濃度：X1、薬剤2の濃度：X2）

data=data.frame(X1=c(rep(0,4),rep(3,4),rep(15,4),rep(35,4)),X2=c(0,3,15,100,0,3,15,100,0,3,15,100,0,3,15,100),CELLN=c(95,86,92,86,90,93,91,94,89,82,88,84,90,91,95,88),CELLS=c(18,22,55,67,14,18,42,80,16,11,37,63,17,35,57,75))

# 特徴量行列
X=as.matrix(cbind(rep(1,nrow(data)),data[,colnames(data) %in% c("X1","X2")],data[,colnames(data) %in% c("X1","X2")]^2))

# y：細胞死亡数、n：総細胞数
y=data$CELLS;n=data$CELLN

# モデル選択　1：ロジット、2：プロビット、3：補対数対数
model=1

# 初期値
alpha=rep(0.0001,(ncol(X)))

# 反復計算回数
ite=10^4

# 学習率
eta=0.001

# 回帰係数保存データ
alpha_beta_data=array(0,dim=c(ite,ncol(X)))

for(j in 1:ite){
  if(model==1){
    p=1/(1+exp(-(X%*%alpha)))
    D=diag(c(1/(n*p*(1-p))))
    V=diag(c(n*p*(1-p)))
  }else{
    if(model==2){
      p=pnorm(X%*%alpha)
      D=diag(c(1/(n*dnorm(X%*%alpha))))
      V=diag(1/c(n*p*(1-p)/((n*dnorm(X%*%alpha))^2)))
    }else{
      p=1-exp(-exp(X%*%alpha))
      V=diag(1/c(p*(log(1-p)^(-2))/(n*(1-p))))
      D=diag(c(1/(-n*(1-p)*log(1-p))))
    }
  }

  mat=solve(t(X)%*%V%*%(X))%*%t(X)%*%V%*%D%*%(y-n*p)
  alpha=alpha+eta*mat
  alpha_beta_data[j,]=alpha
}

res=data.frame(data,pre_freq=n*p,pre_p=p)

# 対数尤度
log_lik=sum(log(dbinom(y,n,p)))

AIC=-2*log_lik+2*(length(alpha))
BIC=-2*log_lik+length(alpha)*log(nrow(X))
mu=n*p

# 残差にあたる部分（尤離度）
D_y_mu1=y*log(y/mu);D_y_mu2=(n-y)*log((n-y)/(n-mu))
D_y_mu1[is.nan(D_y_mu1)>0]=0;D_y_mu2[is.nan(D_y_mu2)>0]=0
D_y_mu=2*D_y_mu1+2*D_y_mu2

# 回帰にあたる部分（尤離度）
D_y_y1=y*log(y/mean(y));D_y_y2=(n-y)*log((n-y)/(n-mean(y)))
D_y_y1[is.nan(D_y_y1)>0]=0;D_y_y2[is.nan(D_y_y2)>0]=0
D_y_y=2*D_y_y1+2*D_y_y2

# 尤離度決定係数
R_dev=1-sum(D_y_mu)/sum(D_y_y)
